@@ -53,6 +53,16 @@
 #define SINGLE_WORKSPACE_MODE     "/backdrop/single-workspace-mode"
 #define SINGLE_WORKSPACE_NUMBER   "/backdrop/single-workspace-number"
 
+#define DESKTOP_ICONS_SHOW_THUMBNAILS        "/desktop-icons/show-thumbnails"
+#define DESKTOP_ICONS_SHOW_NETWORK_REMOVABLE "/desktop-icons/file-icons/show-network-removable"
+#define DESKTOP_ICONS_SHOW_DEVICE_REMOVABLE  "/desktop-icons/file-icons/show-device-removable"
+#define DESKTOP_ICONS_SHOW_UNKNWON_REMOVABLE "/desktop-icons/file-icons/show-unknown-removable"
+#define DESKTOP_ICONS_SHOW_HOME              "/desktop-icons/file-icons/show-home"
+#define DESKTOP_ICONS_SHOW_TRASH             "/desktop-icons/file-icons/show-trash"
+#define DESKTOP_ICONS_SHOW_FILESYSTEM        "/desktop-icons/file-icons/show-filesystem"
+#define DESKTOP_ICONS_SHOW_REMOVABLE         "/desktop-icons/file-icons/show-removable"
+#define DESKTOP_ICONS_MAX_TEMPLATE_FILES     "/desktop-icons/file-icons/max-template-files"
+
 /**
  * File information namespaces queried for #GFileInfo objects.
  */
@@ -76,9 +86,6 @@
 
 G_BEGIN_DECLS
 
-gchar *xfdesktop_backdrop_choose_next(const gchar *filename);
-gchar *xfdesktop_backdrop_choose_random(const gchar *filename);
-
 gboolean xfdesktop_image_file_is_valid(const gchar *filename);
 
 gchar *xfdesktop_get_file_mimetype(const gchar *file);
@@ -86,8 +93,9 @@ gchar *xfdesktop_get_file_mimetype(const gchar *file);
 gboolean xfdesktop_check_is_running(Window *xid);
 void xfdesktop_send_client_message(Window xid, const gchar *msg);
 
-guint xfce_grab_cursor(GtkWidget *w, GdkEventButton *evt);
-gboolean xfdesktop_popup_grab_available(GdkWindow *win, guint32 timestamp);
+gint xfce_translate_image_styles(gint input);
+
+guint32 xfdesktop_popup_keyboard_grab_available(GdkWindow *win);
 
 G_END_DECLS
 
