@@ -245,9 +245,10 @@ xfce_desktop_setup_icon_view(XfceDesktop *desktop)
         xfce_desktop_ensure_system_font_size(desktop);
         
         desktop->priv->icon_view = xfdesktop_icon_view_new(manager);
+        /* If the user set a custom font size, use it. Otherwise use the system
+         * font size */
         xfdesktop_icon_view_set_font_size(XFDESKTOP_ICON_VIEW(desktop->priv->icon_view),
-                                          (!desktop->priv->icons_font_size_set
-                                           || !desktop->priv->icons_font_size)
+                                          (!desktop->priv->icons_font_size_set)
                                           ? desktop->priv->system_font_size
                                           : desktop->priv->icons_font_size);
         if(desktop->priv->icons_size > 0) {
@@ -326,8 +327,14 @@ create_bg_pixmap(GdkScreen *gscreen, gpointer user_data)
 
     g_return_val_if_fail(XFCE_IS_DESKTOP(desktop), NULL);
 
-    if(desktop->priv->workspaces == NULL)
+    /* If the workspaces haven't been created yet there's no need to do the
+     * background pixmap */
+    if(desktop->priv->workspaces == NULL) {
+        DBG("exiting, desktop->priv->workspaces == NULL");
         return NULL;
+    }
+
+    TRACE("really entering");
 
     w = gdk_screen_get_width(gscreen);
     h = gdk_screen_get_height(gscreen);
@@ -359,7 +366,10 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
     GdkRectangle rect;
     GdkRegion *clip_region = NULL;
     gint i, monitor = -1, current_workspace;
-    
+#ifdef G_ENABLE_DEBUG
+    gchar *monitor_name = NULL;
+#endif
+
     TRACE("entering");
     
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
@@ -367,13 +377,6 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
     if(desktop->priv->updates_frozen || !gtk_widget_get_realized(GTK_WIDGET(desktop)))
         return;
 
-    if(!GDK_IS_PIXMAP(pmap)) {
-        pmap = create_bg_pixmap(gscreen, desktop);
-
-        if(!GDK_IS_PIXMAP(pmap))
-            return;
-    }
-
     TRACE("really entering");
 
     current_workspace = xfce_desktop_get_current_workspace(desktop);
@@ -388,9 +391,13 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
     if(monitor == -1)
         return;
 
+#ifdef G_ENABLE_DEBUG
+    monitor_name = gdk_screen_get_monitor_plug_name(gscreen, monitor);
 
-    DBG("backdrop changed for workspace %d, monitor %d (%s)",
-        current_workspace, monitor, gdk_screen_get_monitor_plug_name(gscreen, monitor));
+    DBG("backdrop changed for workspace %d, monitor %d (%s)", current_workspace, monitor, monitor_name);
+
+    g_free(monitor_name);
+#endif
 
     if(xfce_desktop_get_n_monitors(desktop) > 1
        && xfce_workspace_get_xinerama_stretch(desktop->priv->workspaces[current_workspace])) {
@@ -465,9 +472,27 @@ backdrop_changed_cb(XfceBackdrop *backdrop, gpointer user_data)
         /* create the backdrop if needed */
         if(!pix) {
             xfce_backdrop_generate_async(backdrop);
+
+            if(clip_region != NULL)
+                gdk_region_destroy(clip_region);
+
             return;
         }
 
+        /* Create the background pixmap if it isn't already */
+        if(!GDK_IS_PIXMAP(pmap)) {
+            pmap = create_bg_pixmap(gscreen, desktop);
+
+            if(!GDK_IS_PIXMAP(pmap)) {
+                g_object_unref(pix);
+
+                if(clip_region != NULL)
+                    gdk_region_destroy(clip_region);
+
+                return;
+            }
+        }
+
         cr = gdk_cairo_create(GDK_DRAWABLE(pmap));
         gdk_cairo_set_source_pixbuf(cr, pix, rect.x, rect.y);
 
@@ -506,9 +531,6 @@ screen_size_changed_cb(GdkScreen *gscreen, gpointer user_data)
 
     TRACE("entering");
 
-    if(!create_bg_pixmap(gscreen, desktop))
-        return;
-
     current_workspace = xfce_desktop_get_current_workspace(desktop);
 
     if(desktop->priv->nworkspaces <= current_workspace)
@@ -585,6 +607,11 @@ workspace_changed_cb(WnckScreen *wnck_screen,
 
     TRACE("entering");
 
+    /* Ignore workspace changes in single workspace mode so long as we
+     * already have the bg_pixmap loaded */
+    if(xfce_desktop_get_single_workspace_mode(desktop) && desktop->priv->bg_pixmap)
+        return;
+
     current_workspace = desktop->priv->current_workspace;
     new_workspace = xfce_desktop_get_current_workspace(desktop);
 
@@ -603,7 +630,7 @@ workspace_changed_cb(WnckScreen *wnck_screen,
             current_backdrop = xfce_workspace_get_backdrop(desktop->priv->workspaces[current_workspace], i);
             new_backdrop = xfce_workspace_get_backdrop(desktop->priv->workspaces[new_workspace], i);
 
-            if(!xfce_backdrop_compare_backdrops(current_backdrop, new_backdrop)) {
+            if(!xfce_backdrop_compare_backdrops(current_backdrop, new_backdrop) || !desktop->priv->bg_pixmap) {
                 /* only update monitors that require it */
                 backdrop_changed_cb(new_backdrop, user_data);
             }
@@ -644,6 +671,10 @@ workspace_created_cb(WnckScreen *wnck_screen,
                                                                     desktop->priv->property_prefix,
                                                                     nlast_workspace);
 
+    /* Tell workspace whether to cache pixbufs */
+    xfce_workspace_set_cache_pixbufs(desktop->priv->workspaces[nlast_workspace],
+                                     !desktop->priv->single_workspace_mode);
+
     xfce_workspace_monitors_changed(desktop->priv->workspaces[nlast_workspace],
                                     desktop->priv->gscreen);
 
@@ -812,7 +843,7 @@ xfce_desktop_class_init(XfceDesktopClass *klass)
                                     g_param_spec_uint("icon-font-size",
                                                       "icon font size",
                                                       "icon font size",
-                                                      4, 144, 12,
+                                                      0, 144, 12,
                                                       XFDESKTOP_PARAM_FLAGS));
 
     g_object_class_install_property(gobject_class, PROP_ICON_FONT_SIZE_SET,
@@ -848,7 +879,11 @@ xfce_desktop_init(XfceDesktop *desktop)
                                                 XfceDesktopPriv);
     
     gtk_window_set_type_hint(GTK_WINDOW(desktop), GDK_WINDOW_TYPE_HINT_DESKTOP);
-    gtk_window_set_accept_focus(GTK_WINDOW(desktop), FALSE);
+    /* Accept focus is needed for the menu pop up either by the menu key on
+     * the keyboard or Shift+F10. */
+    gtk_window_set_accept_focus(GTK_WINDOW(desktop), TRUE);
+    /* Can focus is needed for the gtk_grab_add/remove commands */
+    gtk_widget_set_can_focus(GTK_WIDGET(desktop), TRUE);
     gtk_window_set_resizable(GTK_WINDOW(desktop), FALSE);
 }
 
@@ -856,7 +891,7 @@ static void
 xfce_desktop_finalize(GObject *object)
 {
     XfceDesktop *desktop = XFCE_DESKTOP(object);
-    
+
     g_object_unref(G_OBJECT(desktop->priv->channel));
     g_free(desktop->priv->property_prefix);
 
@@ -1007,17 +1042,15 @@ xfce_desktop_realize(GtkWidget *widget)
     wnck_screen = wnck_screen_get(gdk_screen_get_number(desktop->priv->gscreen));
     desktop->priv->wnck_screen = wnck_screen;
 
+    /* Watch for single workspace setting changes */
     xfconf_g_property_bind(desktop->priv->channel,
                            SINGLE_WORKSPACE_MODE, G_TYPE_BOOLEAN,
                            G_OBJECT(desktop), "single-workspace-mode");
-
     xfconf_g_property_bind(desktop->priv->channel,
                            SINGLE_WORKSPACE_NUMBER, G_TYPE_INT,
                            G_OBJECT(desktop), "single-workspace-number");
 
-    /* Start with an invalid workspace so it updates */
-    desktop->priv->current_workspace = -1;
-
+    /* watch for workspace changes */
     g_signal_connect(desktop->priv->wnck_screen, "active-workspace-changed",
                      G_CALLBACK(workspace_changed_cb), desktop);
     g_signal_connect(desktop->priv->wnck_screen, "workspace-created",
@@ -1025,8 +1058,7 @@ xfce_desktop_realize(GtkWidget *widget)
     g_signal_connect(desktop->priv->wnck_screen, "workspace-destroyed",
                      G_CALLBACK(workspace_destroyed_cb), desktop);
 
-    xfce_desktop_monitors_changed(desktop->priv->gscreen, desktop);
-    
+    /* watch for screen changes */
     g_signal_connect(G_OBJECT(desktop->priv->gscreen), "size-changed",
             G_CALLBACK(screen_size_changed_cb), desktop);
     g_signal_connect(G_OBJECT(desktop->priv->gscreen), "composited-changed",
@@ -1050,14 +1082,13 @@ xfce_desktop_unrealize(GtkWidget *widget)
     gchar property_name[128];
     
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
-    
-    if(gtk_major_version > 2
-       || (gtk_major_version == 2 && gtk_minor_version >= 13))
-    {
-        g_signal_handlers_disconnect_by_func(G_OBJECT(desktop->priv->gscreen),
-                                             G_CALLBACK(xfce_desktop_monitors_changed),
-                                             desktop);
-    }
+
+    /* disconnect all the xfconf settings to this desktop */
+    xfconf_g_property_unbind_all(G_OBJECT(desktop));
+
+    g_signal_handlers_disconnect_by_func(G_OBJECT(desktop->priv->gscreen),
+                                         G_CALLBACK(xfce_desktop_monitors_changed),
+                                         desktop);
     
     if(gtk_widget_get_mapped(widget))
         gtk_widget_unmap(widget);
@@ -1111,21 +1142,6 @@ xfce_desktop_unrealize(GtkWidget *widget)
     gtk_widget_set_realized(widget, FALSE);
 }
 
-static void
-xfce_desktop_release_grab(GtkWidget *w)
-{
-    XfceDesktop *desktop = XFCE_DESKTOP(w);
-    GdkDisplay  *display;
-
-    g_return_if_fail(XFCE_IS_DESKTOP(w));
-
-    if(desktop->priv->grab_time != 0) {
-        display = gdk_screen_get_display(desktop->priv->gscreen);
-        gdk_display_pointer_ungrab(display, desktop->priv->grab_time);
-        desktop->priv->grab_time = 0;
-    }
-}
-
 static gboolean
 xfce_desktop_button_press_event(GtkWidget *w,
                                 GdkEventButton *evt)
@@ -1134,37 +1150,32 @@ xfce_desktop_button_press_event(GtkWidget *w,
     guint state = evt->state;
     XfceDesktop *desktop = XFCE_DESKTOP(w);
 
+    TRACE("entering");
+
     g_return_val_if_fail(XFCE_IS_DESKTOP(w), FALSE);
 
     if(evt->type == GDK_BUTTON_PRESS) {
         if(button == 3 || (button == 1 && (state & GDK_SHIFT_MASK))) {
 #ifdef ENABLE_DESKTOP_ICONS
+            /* Let the icon view handle these menu pop ups */
             if(desktop->priv->icons_style != XFCE_DESKTOP_ICON_STYLE_NONE)
                 return FALSE;
 #endif
-            if(desktop->priv->grab_time == 0)
-                desktop->priv->grab_time = xfce_grab_cursor(w, evt);
-            else
-                return FALSE;
+            /* no icons on the desktop, grab the focus and pop up the menu */
+            if(!gtk_widget_has_grab(w))
+                gtk_grab_add(w);
 
-            if(desktop->priv->grab_time != 0) {
-                xfce_desktop_popup_root_menu(desktop, button,
-                                             desktop->priv->grab_time);
+                xfce_desktop_popup_root_menu(desktop, button, evt->time);
                 return TRUE;
-            }
         } else if(button == 2 || (button == 1 && (state & GDK_SHIFT_MASK)
                                   && (state & GDK_CONTROL_MASK)))
         {
-            if(desktop->priv->grab_time == 0)
-                desktop->priv->grab_time = xfce_grab_cursor(w, evt);
-            else
-                return FALSE;
+            /* always grab the focus and pop up the menu */
+            if(!gtk_widget_has_grab(w))
+                gtk_grab_add(w);
 
-            if(desktop->priv->grab_time != 0) {
-                xfce_desktop_popup_secondary_root_menu(desktop, button,
-                                                       desktop->priv->grab_time);
-                return TRUE;
-            }
+            xfce_desktop_popup_secondary_root_menu(desktop, button, evt->time);
+            return TRUE;
         }
     }
     
@@ -1175,17 +1186,23 @@ static gboolean
 xfce_desktop_button_release_event(GtkWidget *w,
                                   GdkEventButton *evt)
 {
-    xfce_desktop_release_grab(w);
+    TRACE("entering");
 
-    return TRUE;
+    gtk_grab_remove(w);
+
+    return FALSE;
 }
 
+/* This function gets called when the user presses the menu key on the keyboard.
+ * Or Shift+F10 or whatever key binding the user has chosen. */
 static gboolean
 xfce_desktop_popup_menu(GtkWidget *w)
 {
     GdkEventButton *evt;
     guint button, etime;
-    
+
+    TRACE("entering");
+
     evt = (GdkEventButton *)gtk_get_current_event();
     if(evt && GDK_BUTTON_PRESS == evt->type) {
         button = evt->button;
@@ -1486,6 +1503,8 @@ static void
 xfce_desktop_set_single_workspace_mode(XfceDesktop *desktop,
                                        gboolean single_workspace)
 {
+    gint i;
+
     g_return_if_fail(XFCE_IS_DESKTOP(desktop));
 
     if(single_workspace == desktop->priv->single_workspace_mode)
@@ -1495,8 +1514,18 @@ xfce_desktop_set_single_workspace_mode(XfceDesktop *desktop,
 
     DBG("single_workspace_mode now %s", single_workspace ? "TRUE" : "FALSE");
 
-    /* Fake a screen size changed to update the backdrop */
-    screen_size_changed_cb(desktop->priv->gscreen, desktop);
+    /* update the workspaces & backdrops if they should cache their pixbuf */
+    if(desktop->priv->workspaces) {
+        for(i = 0; i < desktop->priv->nworkspaces; i++) {
+            /* set cache to TRUE when single_workspace is FALSE */
+            xfce_workspace_set_cache_pixbufs(desktop->priv->workspaces[i], !single_workspace);
+        }
+    }
+
+    /* If the desktop has been realized then fake a screen size change to
+     * update the backdrop. There's no reason to if there's no desktop yet */
+    if(gtk_widget_get_realized(GTK_WIDGET(desktop)))
+        screen_size_changed_cb(desktop->priv->gscreen, desktop);
 }
 
 static void
@@ -1586,11 +1615,12 @@ xfce_desktop_do_menu_popup(XfceDesktop *desktop,
 
     gtk_menu_attach_to_widget(GTK_MENU(menu), GTK_WIDGET(desktop), NULL);
 
-    /* bug #3652: for some reason passing the correct button here breaks
-     * on some systems but not others.  always pass 0 for now. */
-    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, activate_time);
+    /* Per gtk_menu_popup's documentation "for conflict-resolve initiation of
+     * concurrent requests for mouse/keyboard grab requests." */
+    if(activate_time == 0)
+        activate_time = gtk_get_current_event_time();
 
-    xfce_desktop_release_grab(GTK_WIDGET(desktop));
+    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button, activate_time);
 }
 
 void
@@ -1598,24 +1628,16 @@ xfce_desktop_popup_root_menu(XfceDesktop *desktop,
                              guint button,
                              guint activate_time)
 {
-    GdkScreen *screen;
+    TRACE("entering");
 
-    if(gtk_widget_has_screen(GTK_WIDGET(desktop)))
-        screen = gtk_widget_get_screen(GTK_WIDGET(desktop));
-    else
-        screen = gdk_display_get_default_screen(gdk_display_get_default());
-
-    if(activate_time == 0) {
-        activate_time = GDK_CURRENT_TIME;
-        if(!xfdesktop_popup_grab_available(gdk_screen_get_root_window(screen),
-                                           activate_time)) {
-            g_critical("keyboard or mouse grab failed.");
-            return;
-        }
-    }
+    /* If it's launched by xfdesktop --menu we won't have an event time.
+     * Grab the keyboard focus */
+    if(activate_time == 0)
+        activate_time = xfdesktop_popup_keyboard_grab_available(gtk_widget_get_window(GTK_WIDGET(desktop)));
 
     xfce_desktop_do_menu_popup(desktop, button, activate_time,
                                signals[SIG_POPULATE_ROOT_MENU]);
+
 }
 
 void
@@ -1623,21 +1645,10 @@ xfce_desktop_popup_secondary_root_menu(XfceDesktop *desktop,
                                        guint button,
                                        guint activate_time)
 {
-    GdkScreen *screen;
-
-    if(gtk_widget_has_screen(GTK_WIDGET(desktop)))
-        screen = gtk_widget_get_screen(GTK_WIDGET(desktop));
-    else
-        screen = gdk_display_get_default_screen(gdk_display_get_default());
-
-    if(activate_time == 0) {
-        activate_time = GDK_CURRENT_TIME;
-        if(!xfdesktop_popup_grab_available(gdk_screen_get_root_window(screen),
-                                           activate_time)) {
-            g_critical("keyboard or mouse grab failed.");
-            return;
-        }
-    }
+    /* If it's launched by xfdesktop --windowlist we won't have an event time.
+     * Grab the keyboard focus */
+    if(activate_time == 0)
+        activate_time = xfdesktop_popup_keyboard_grab_available(gtk_widget_get_window(GTK_WIDGET(desktop)));
 
     xfce_desktop_do_menu_popup(desktop, button, activate_time,
                                signals[SIG_POPULATE_SECONDARY_ROOT_MENU]);
